#version 330

out vec4 final_color;

in vec3 VS_normal_ws;
in vec3 VS_position_ws;
in vec2 VS_tex_coord;

uniform vec3 material_ambient_color;
uniform vec3 material_diffuse_color;
uniform vec3 material_specular_color;
uniform float material_shininess;

uniform vec4 light_position;
uniform vec3 light_ambient_color;
uniform vec3 light_diffuse_color;
uniform vec3 light_specular_color;

uniform vec3 eye_position;

void main()
{
    const vec3 red = vec3(1.0, 0.3, 0.0);
    const vec3 white = vec3(1.0, 1.0, 1.0);

    // Bricks
    /*
    // Number of bricks per row and column
    vec2 repeat = vec2(4.0, 8.0);
    // Offsetting every other row by half a brick
    vec2 offset = vec2(0.5, 0.0) * mod(floor(VS_tex_coord.t * repeat.t), 2);
    // Coordinates inside the currently drawn brick
    vec2 brick_coord = fract(VS_tex_coord * repeat + offset);
    // Deciding whether this fragment represents a brick or the mortar
    vec2 brick = smoothstep(0.1, 0.15, brick_coord)
            - smoothstep(0.95, 1.0, brick_coord);
    // Combining vertical and horisontal stripes
    vec3 tex_color = mix(white, red, brick.s * brick.t);
    */


    // "Granko"
    // Number of dots per row and column
    vec2 repeat = vec2(5.0);
    // Offsetting every other row by half a dot
    vec2 offset = vec2(0.5, 0.0) * floor(VS_tex_coord.t * repeat.t);
    // Coordinates inside the currently drawn dot
    vec2 granko_coord = fract(VS_tex_coord * repeat + offset);
    // Deciding whether this fragment is inside the dot or outside
    float granko = smoothstep (0.28, 0.3, distance(granko_coord, vec2(0.5)));
    // Dot is red, outside is white
    vec3 tex_color = mix(red, white, granko);

    // Light (see third lesson if you do not understand something)
    vec3 N = normalize(VS_normal_ws);
    vec3 Eye = normalize(eye_position - VS_position_ws);

    vec3 L = normalize(light_position.xyz - VS_position_ws * light_position.w);

    vec3 H = normalize(L + Eye);

    float Idiff = max(dot(N, L), 0.0);
    float Ispec = Idiff * pow(max(dot(N, H), 0.0), material_shininess);

    vec3 mat_ambient = material_ambient_color * tex_color;
    vec3 mat_diffuse = material_diffuse_color * tex_color;
    vec3 mat_specular = material_specular_color;

    vec3 light =
        mat_ambient * light_ambient_color +
        mat_diffuse * light_diffuse_color * Idiff +
        mat_specular * light_specular_color * Ispec;

    final_color = vec4(light, 1.0);
}
